/*	zc001.c

	CPI-ZC001 control.

	Copyright (C) CPI Technologies, Inc.
*/

#pragma	warning( disable: 4115 4201 4214 4514 )
#include <windows.h>
#pragma	warning( 1: 4115 4201 4214 )
#include <tchar.h>

#include <assert.h>

#include "resource.h"

#include "tchars.h"
#include "elog.h"
#include "commsel.h"
#include "cpizxio.h"
#include "zx.h"

/*//////////////////////////////////////////////////////////////////////
*/

#define	LENGTH_OF( ar )	( sizeof ( ar ) / sizeof ( *ar ) )

#define	DSCT( id, st )	static TCHAR const id[] = _T( st )

/*//////////////////////////////////////////////////////////////////////
*/

typedef	struct
{
	HICON	hicSm;		/*    icon: 16. */
	HCURSOR	hcuNorm;	/*  cursor: normal. */
	HCURSOR	hcuWait;	/*  cursor: wait. */

	HWND	hw0;		/*  window: parent. */
	HWND	hd;		/*  dialog: main. */

	HWND	hbSel;		/*  button: select communication port. */
	HWND	hlDevs;		/* listbox: units. */
	HWND	hbOpen;		/*  button: open unit. */

	HELOG	hl;		/* log. */

	HZXC	hz;		/* coordinator. */
	int	iC;		/* communication port #. */
	UINT	nU;		/* # of units. */

				/* initial window title. */
	TCHAR	sz0Capt[ 1 + _MAX_PATH ];
}	TDLG;

/*//////////////////////////////////////////////////////////////////////
*/

static	INT_PTR CALLBACK cb_dlg( HWND hd, UINT uM, WPARAM wP, LPARAM lP );

/*//////////////////////////////////////////////////////////////////////
*/

DSCT( s_szOkLn,		"ok.\n" );
DSCT( s_szNgLn,		"NG.\n" );
DSCT( s_szErrorDLn,	"ERROR %d.\n" );

/*//////////////////////////////////////////////////////////////////////
*/

TDLG s_v;

/*//////////////////////////////////////////////////////////////////////
*/

void cur_wait( void )
{
	if ( NULL != s_v.hcuWait )
	{
		( void )SetCursor( s_v.hcuWait );
	}
}

/*//////////////////////////////////////////////////////////////////////
*/

void cur_norm( void )
{
	if ( NULL != s_v.hcuNorm )
	{
		( void )SetCursor( s_v.hcuNorm );
	}
}

/*//////////////////////////////////////////////////////////////////////
*/

static	BOOL on_close( HWND hd );

/*//////////////////////////////////////////////////////////////////////
*/

DSCT( s_szCap,		"CPI-ZC001" );

/*//////////////////////////////////////////////////////////////////////
*/

HWND zc001( HWND hw0 )
{
DSCT( s_szFEGetModuleHandle,
			"GetModuleHandle failed.\r\nError: %ld"
			);
DSCT( s_szFELoadImage,	"LoadImage failed.\r\nError: %ld" );
DSCT( s_szFELoadCursor,	"LoadCursor failed.\r\nError: %ld" );
DSCT( s_szFECreateDialog,
			"CreateDialog failed.\r\nError: %ld"
			);
	HINSTANCE	hi;	/* current instance. */
	HWND	hd;	/* dialog. */
	LPCTSTR	pszFE;	/* error message format. */

	s_v.hw0 = hw0;

	pszFE = s_szFEGetModuleHandle;

	for ( ; ; )			/* [vł܂B */
	{
		hi = GetModuleHandle( NULL );
		if ( NULL == hi )
		{
			break;
		}

		s_v.hicSm = LoadImage(
				hi,
				MAKEINTRESOURCE( IDI_MAIN ),
				IMAGE_ICON,
				GetSystemMetrics( SM_CXSMICON ),
				GetSystemMetrics( SM_CYSMICON ),
				LR_SHARED	/* destroyed automatically. */
				);
		if ( NULL == s_v.hicSm )
		{
			pszFE = s_szFELoadImage;

			break;
		}

		pszFE = s_szFELoadCursor;

		s_v.hcuNorm = LoadCursor( NULL, IDC_ARROW );
		if ( NULL == s_v.hcuNorm )
		{
			break;
		}

		s_v.hcuWait = LoadCursor( NULL, IDC_WAIT );
		if ( NULL == s_v.hcuWait )
		{
			break;
		}

		hd = CreateDialog(
				hi,
				MAKEINTRESOURCE( IDD_ZC001 ),
				hw0,
				cb_dlg
				);
		if ( NULL == hd )
		{
			pszFE = s_szFECreateDialog;

			break;
		}

		( void )ShowWindow( hd, SW_NORMAL );

		return hd;
	}

	( void )mbox(
			hw0,
			MB_OK | MB_ICONEXCLAMATION,
			s_szCap,
			pszFE,
			GetLastError()
			);

	( void )on_close( NULL );

	return NULL;
}

/*//////////////////////////////////////////////////////////////////////
*/

static	BOOL on_cmnd( HWND hd, WPARAM wP, LPARAM lP );
static	BOOL on_init( HWND hd );
static	void on_devchg( void );

/*//////////////////////////////////////////////////////////////////////
*/

static	INT_PTR CALLBACK cb_dlg( HWND hd, UINT uM, WPARAM wP, LPARAM lP )
{
	switch ( uM )
	{
	default:
		return FALSE;		/* message not handled. */

	case WM_CLOSE:
		return on_close( hd );

	case WM_COMMAND:
		return on_cmnd( hd, wP, lP );

	case WM_INITDIALOG:
		return on_init( hd );

	case WM_DEVICECHANGE:
		on_devchg();
		break;
	}

	return TRUE;			/* message handled. */
}

/*//////////////////////////////////////////////////////////////////////
*/

static	BOOL on_close( HWND hd )
{
	if ( NULL != s_v.hl )
	{
		( void )el_clo( s_v.hl );
		s_v.hl = NULL;
	}

	if ( NULL != hd )
	{
		( void )DestroyWindow( hd );
		s_v.hd = NULL;
	}

	if ( NULL != s_v.hicSm )
	{
		( void )DestroyIcon( s_v.hicSm );
		s_v.hicSm = NULL;
	}

	if ( NULL != s_v.hw0 )
	{
		( void )PostMessage( s_v.hw0, WM_CLOSE, 0, 0 );
		s_v.hw0 = NULL;
	}

	return TRUE;			/* message handled. */
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void close_all( void )
{
	int	ri;	/* result. */

	if ( NULL == s_v.hz )
	{
		return;
	}

	( void )EnableWindow( s_v.hbOpen, FALSE );

	( void )SendMessage( s_v.hlDevs, LB_RESETCONTENT, 0, 0 );

	( void )PostMessage( s_v.hw0, WM_X_ZAP, 0, 0 );

	el_putf( s_v.hl, _T( "ZxClose %d ... " ), s_v.iC );
	ri = ZxClose( s_v.hz );
	if ( ri )
	{
		el_putf( s_v.hl, s_szErrorDLn, ri );

		return;
	}
	s_v.hz = NULL;
	el_putf( s_v.hl, s_szOkLn );

	s_v.iC = 0;
	s_v.nU = 0;
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_end( HWND hd )
{
	close_all();

	( void )PostMessage( hd, WM_CLOSE, 0, 0 );
}

/*//////////////////////////////////////////////////////////////////////
*/

#define	CMS_TMO			8000	/* timeout [mS]. */

static	void do_enum( UINT iC )
{
	int	nU;	/* # of units. */
			/* */
	TCHAR	sz[ 1 + _MAX_PATH ];
	int	ri;	/* result. */

	el_putf( s_v.hl, _T( "ZxOpen %d ... " ), iC );
	s_v.hz = ZxOpen( iC, CMS_TMO );
	if ( NULL == s_v.hz )
	{
		el_putf( s_v.hl, s_szNgLn );

		return;
	}
	el_putf( s_v.hl, s_szOkLn );

	el_putf( s_v.hl, _T( "ZxIsZc ... " ) );
	if ( ! ZxIsZc( s_v.hz ) )
	{
		el_putf( s_v.hl, _T( "NOT COORDINATOR.\n" ) );

		return;
	}
	el_putf( s_v.hl, s_szOkLn );

	el_putf( s_v.hl, _T( "ZcEnum ... " ) );
	nU = ZcEnum( s_v.hz );
	if ( 0 > nU )
	{
		el_putf( s_v.hl, s_szErrorDLn, nU );

		return;
	}
	if ( 1 > nU )
	{
		el_putf( s_v.hl, _T( "NO UNITS.\n" ) );

		el_putf( s_v.hl, _T( "ZxClose %d ... " ), iC );
		ri = ZxClose( s_v.hz );
		if ( ri )
		{
			el_putf( s_v.hl, s_szErrorDLn, ri );

			return;
		}
		s_v.hz = NULL;
		el_putf( s_v.hl, s_szOkLn );

		return;
	}
	el_putf( s_v.hl, _T( "%d units found.\n" ), nU );
	assert( nU <= ZXC_MAX_DEVS );

{
	HZXN	hv;	/* unit information. */
	UINT	nA;	/* # of added. */
	BYTE	bV;	/* unit select switch value. */
	int	ii;	/* iterator. */

	nA = 0;
	for ( ii = 0; ii < nU; ii++ )
	{
		hv = ZcDev( s_v.hz, ( UINT )ii );
		if ( NULL == hv )
		{
			continue;
		}

		( void )_stprintf_s(
				sz,
				LENGTH_OF( sz ),
				_T( "%hs %08lX-%08lX\n" ),
				hv->ni,
				hv->sh,
				hv->sl
				);
		el_putf( s_v.hl, sz );

		if ( ! ZnIsZd132( hv )
				&& ! ZnIsZd232( hv )
				&& ! ZnIsZd332( hv )
				)
		{
DSCT( s_szME,	"NOT A CPI-ZD.\n" );

			el_putf( s_v.hl, s_szME );

			continue;
		}

		if ( ZXR_OK != ZdSwtGet( hv, &bV ) )
		{
DSCT( s_szME,	"MAY BE CPI-UD OR CONFIGURATION MODE.\n" );

			el_putf( s_v.hl, s_szME );

			continue;
		}

		( void )_stprintf_s(
				sz,
				LENGTH_OF( sz ),
				_T( "%hs sw:%02X %08lX-%08lX" ),
				hv->ni,
				bV,
				hv->sh,
				hv->sl
				);

		( void )SendMessage(
				s_v.hlDevs,
				LB_ADDSTRING,
				0,
				( LPARAM )sz
				);

		( void )SendMessage(
				s_v.hlDevs,
				LB_SETITEMDATA,
				nA,
				ii
				);

		nA++;
	}
}

	s_v.iC = iC;
	s_v.nU = ( UINT )nU;

	( void )_stprintf_s(
			sz,
			LENGTH_OF( sz ),
			_T( "%s COM%d" ),
			s_v.sz0Capt,
			iC
			);
	( void )SetWindowText( s_v.hd, sz );

	( void )EnableWindow( s_v.hbOpen, 0 < nU );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_sel( HWND hd )
{
	int	iC;	/* communication port #. */

	iC = unit_sel( hd );
	if ( 0 > iC )
	{
		el_putf( s_v.hl,
				_T( "coodinator selection canceled.\n" )
				);

		return;
	}

	( void )SetCursor( s_v.hcuWait );

	close_all();

	do_enum( iC );

	( void )SetCursor( s_v.hcuNorm );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_open( HWND hd )
{
	HZXN	hv;	/* target device. */
	int	iU;	/* unit #. */

	iU = ( int )SendMessage( s_v.hlDevs, LB_GETCURSEL, 0, 0 );
	if ( 0 > iU )
	{
		return;
	}

	iU = ( int )SendMessage( s_v.hlDevs, LB_GETITEMDATA, iU, 0 );
	if ( 0 > iU )
	{
		return;
	}

	hv = ZcDev( s_v.hz, iU );
	if ( NULL == hv )
	{
		( void )mbox(
				hd,
				MB_OK | MB_ICONEXCLAMATION,
				s_szCap,
				_T( "Device not found." )
				);

		return;
	}

	( void )PostMessage( s_v.hw0, WM_X_OPEN, 0, ( LPARAM )hv );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	BOOL on_cmnd( HWND hd, WPARAM wP, LPARAM lP )
{
	( void )lP;

	switch ( wP )
	{
	default:
		return FALSE;		/* message not handled. */

	case MAKELONG( IDOK, BN_CLICKED ):
	case MAKELONG( IDCANCEL, BN_CLICKED ):
		cmnd_end( hd );
		break;

	case MAKELONG( IDC_BUTN_COMM_SEL, BN_CLICKED ):
		cmnd_sel( hd );
		break;

	case MAKELONG( IDC_BUTN_OPEN, BN_CLICKED ):
	case MAKELONG( IDC_LIST_UNITS, LBN_DBLCLK ):
		cmnd_open( hd );
		break;

	case MAKELONG( IDC_BUTN_LOG_CLEAR, BN_CLICKED ):
		el_clr( s_v.hl );
		break;
	}

	return TRUE;			/* message handled. */
}

/*//////////////////////////////////////////////////////////////////////
*/

static	BOOL on_init( HWND hd )
{
	assert( NULL != hd );
	assert( NULL != s_v.hicSm );

	s_v.hd = hd;

					/* title bar, etc. */
	( void )SendMessage(
			hd,
			WM_SETICON,
			ICON_SMALL,
			( LPARAM )s_v.hicSm
			);

	s_v.hbSel = GetDlgItem( hd, IDC_BUTN_COMM_SEL );

	s_v.hlDevs = GetDlgItem( hd, IDC_LIST_UNITS );

	s_v.hbOpen = GetDlgItem( hd, IDC_BUTN_OPEN );
	( void )EnableWindow( s_v.hbOpen, FALSE );

	s_v.hl = el_cat( GetDlgItem( hd, IDC_EDIT_LOG ), 12 );

	( void )GetWindowText(
			s_v.hd,
			s_v.sz0Capt,
			LENGTH_OF( s_v.sz0Capt )
			);

	return TRUE;			/* set default focus. */
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void on_devchg( void )
{
	if ( NULL != s_v.hl )
	{
		el_putf( s_v.hl, _T( "WM_DEVICECHANGE encountered.\n" ) );
	}

	if ( NULL == s_v.hz )
	{
		return;
	}

	if ( ! ZxReady( s_v.hz ) )
	{
		if ( NULL != s_v.hl )
		{
			el_putf( s_v.hl, _T( "SURPRISE REMOVAL.\n" ) );
		}

		close_all();
	}
}
