/*	zdx32.c

	CPI-ZDx32 control.

	Copyright (C) CPI Technologies, Inc.
*/

#pragma	warning( disable: 4115 4201 4214 4514 )
#include <windows.h>
#pragma	warning( 1: 4115 4201 4214 )
#include <tchar.h>

#include <assert.h>

#include "resource.h"

#include "tchars.h"
#include "edit16.h"
#include "elog.h"
#include "cpizxio.h"
#include "zx.h"

/*//////////////////////////////////////////////////////////////////////
*/

#define	LENGTH_OF( ar )	( sizeof ( ar ) / sizeof ( *ar ) )

#define	DSCT( id, st )	static TCHAR const id[] = _T( st )

/*======================================================================
*/

#ifdef	_M_IX86	/*[*/

#define	WL_USERDATA	GWL_USERDATA
#define	WLGet		GetWindowLong
#define	WLSet		SetWindowLong

#else	/*][*/

#define	WL_USERDATA	GWLP_USERDATA
#define	WLGet		GetWindowLongPtr
#define	WLSet		SetWindowLongPtr

#endif	/*]*/

/*//////////////////////////////////////////////////////////////////////
*/

typedef	struct
{
	HWND	hg;		/* groupbox: port. */
	HWND	hbIn;		/*   button: input. */
	HWND	heIn;		/*     edit: input value. */
	HWND	heOut;		/*     edit: output value. */
	HWND	hbOut;		/*   button: output. */
struct	/* monitor. */
{
	HWND	hg;		/* groupbox: monitor. */
	HWND	hsMode;		/*   static: mode. */
	HWND	hcMode;		/* combobox: mode. */
	HWND	hsMask;		/*   static: mask. */
	HWND	heMask;		/* combobox: mask. */
	HWND	hsComp;		/*   static: data. */
	HWND	heComp;		/* combobox: data. */
	HWND	hbSet;		/*   button: set. */
	HWND	hbStat;		/*   button: get status. */
	HWND	hsStat;		/*   static: status. */
	HWND	heStat;		/*     edit: status value. */
	HWND	hsData;		/*   static: data. */
	HWND	heData;		/*     edit: data value. */
}	mon;
struct	/* WDT. */
{
	HWND	hg;		/* groupbox: monitor. */
	HWND	hsOut;		/*   static: output. */
	HWND	heOut;		/*     edit: output value. */
	HWND	hbSet;		/*   button: set. */
}	wdt;
}	TPORT;

typedef	TPORT FAR*	LPPORT;
typedef	TPORT const FAR*	LPCPORT;

typedef	struct
{
	HWND	hw0;		/*   window: parent. */
	HWND	hd;		/*   dialog: main. */

	HWND	hbNop;		/*   button: NOP. */
	HWND	hbErr;		/*   button: get last error. */
	HWND	hbName;		/*   button: get unit name. */
	HWND	hbVer;		/*   button: get version. */
	HWND	hbType;		/*   button: get unit type. */
	HWND	hbSwt;		/*   button: get switch value. */
	HWND	hbAtdb;		/*   button: ATDB. */

struct	/* common. */
{
	HWND	hg;		/* groupbox. */
struct	/* filter. */
{
	HWND	hg;		/* groupbox: filter. */
	HWND	hsFreq;		/*   static: interval. */
	HWND	hcFreq;		/* combobox: interval. */
	HWND	hsStep;		/*   static: steps. */
	HWND	hcStep;		/* combobox: steps. */
	HWND	hbSet;		/*   button: set. */
}	filt;
struct	/* WDT. */
{
	HWND	hg;		/* groupbox: filter. */
	HWND	hsTmo;		/*   static: interval. */
	HWND	hcTmo;		/* combobox: interval. */
	HWND	hbSet;		/*   button: set. */
	HWND	hbRst;		/*   button: reset. */
	HWND	hbStat;		/*   button: get status. */
	HWND	heStat;		/*     edit: status. */
	HWND	hbClr;		/*   button: clear. */
}	wdt;
}	comn;

				/* port controls. */
	TPORT	ap[ ZD_NUM_PORTS ];

	HELOG	hl;		/* log. */

	HZXN	hv;		/* device. */
}	TZDX32;

typedef	TZDX32 FAR*	LPZDX32;

/*//////////////////////////////////////////////////////////////////////
*/

static	INT_PTR CALLBACK cb_dlg( HWND hd, UINT uM, WPARAM wP, LPARAM lP );

/*//////////////////////////////////////////////////////////////////////
*/

DSCT( s_sz00,		"00" );
DSCT( s_szFF,		"FF" );
DSCT( s_szF_d,		"%d" );
DSCT( s_szF_02X,	"%02X" );

/*//////////////////////////////////////////////////////////////////////
*/

static	struct
{
				/* units. */
	TZDX32	au[ ZXC_MAX_DEVS ];
}	s_v;

/*//////////////////////////////////////////////////////////////////////
in	hw0	parent window.
	hv	device.
*/

HWND zdx32( HWND hw0, HZXN hv )
{
DSCT( s_szCap,	"CPI-ZDx32" );
DSCT( s_szEHz,	"Invalid coordinator control handle." );
DSCT( s_szEIx,	"Invalid device number." );
DSCT( s_szFECreateDialog,
		"CreateDialogParam failed.\r\nError: %ld"
		);
	LPZDX32	pu;	/* unit. */
	HWND	hd;	/* dialog. */

	if ( NULL == hv )
	{
		( void )mbox(
				hw0,
				MB_OK | MB_ICONEXCLAMATION,
				s_szCap,
				s_szEHz
				);

		return NULL;
	}

	cur_wait();

	pu = hv->ix + s_v.au;
	pu->hw0 = hw0;
	pu->hv = hv;

	hd = CreateDialogParam(
			GetModuleHandle( NULL ),
			MAKEINTRESOURCE( IDD_ZDX32 ),
			hw0,
			cb_dlg,
			( LPARAM )pu
			);
	if ( NULL == hd )
	{
				/* for message box. */
		TCHAR	szCap[ 1 + 255 ];

		( void )_stprintf_s(
				szCap,
				LENGTH_OF( szCap ),
				_T( "%hs" ),
				hv->ni
				);

		( void )mbox(
				hw0,
				MB_OK | MB_ICONEXCLAMATION,
				szCap,
				s_szFECreateDialog,
				GetLastError()
				);

		return NULL;
	}

	( void )ShowWindow( hd, SW_NORMAL );

	return hd;
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void on_close( HWND hd );
static	BOOL on_cmnd( HWND hd, WPARAM wP, LPARAM lP );
static	BOOL on_init( HWND hd, LPZDX32 pu );

/*//////////////////////////////////////////////////////////////////////
*/

static	INT_PTR CALLBACK cb_dlg( HWND hd, UINT uM, WPARAM wP, LPARAM lP )
{
	switch ( uM )
	{
	default:
		return FALSE;		/* message not handled. */

	case WM_CLOSE:
		on_close( hd );
		break;

	case WM_COMMAND:
		return on_cmnd( hd, wP, lP );

	case WM_INITDIALOG:
		return on_init( hd, ( LPZDX32 )lP );
	}

	return TRUE;			/* message handled. */
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void on_close( HWND hd )
{
	LPZDX32	pu;	/* device information. */

	/*
	// obtain my device information.
	*/

	pu = ( LPZDX32 )WLGet( hd, WL_USERDATA );
	assert( NULL != pu );

	( void )el_clo( pu->hl );

	assert( NULL != pu->hd );
	( void )DestroyWindow( pu->hd );
	pu->hd = NULL;

	assert( NULL != pu->hw0 );
	( void )PostMessage( pu->hw0, WM_X_CLOSE, pu->hv->ix, 0 );
	pu->hw0 = NULL;
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void ena_all( LPZDX32 pu, BOOL fEna )
{
	( void )EnableWindow( pu->hbNop, fEna );
	( void )EnableWindow( pu->hbErr, fEna );
	( void )EnableWindow( pu->hbName, fEna );
	( void )EnableWindow( pu->hbVer, fEna );
	( void )EnableWindow( pu->hbType, fEna );
	( void )EnableWindow( pu->hbSwt, fEna );
	( void )EnableWindow( pu->hbAtdb, fEna );

	( void )EnableWindow( pu->comn.hg, fEna );

	( void )EnableWindow( pu->comn.filt.hg, fEna );
	( void )EnableWindow( pu->comn.filt.hsFreq, fEna );
	( void )EnableWindow( pu->comn.filt.hcFreq, fEna );
	( void )EnableWindow( pu->comn.filt.hsStep, fEna );
	( void )EnableWindow( pu->comn.filt.hcStep, fEna );
	( void )EnableWindow( pu->comn.filt.hbSet, fEna );

	( void )EnableWindow( pu->comn.wdt.hg, fEna );
	( void )EnableWindow( pu->comn.wdt.hsTmo, fEna );
	( void )EnableWindow( pu->comn.wdt.hcTmo, fEna );
	( void )EnableWindow( pu->comn.wdt.hbSet, fEna );
	( void )EnableWindow( pu->comn.wdt.hbRst, fEna );
	( void )EnableWindow( pu->comn.wdt.hbStat, fEna );
	( void )EnableWindow( pu->comn.wdt.heStat, fEna );
	( void )EnableWindow( pu->comn.wdt.hbClr, fEna );

{
	LPPORT	ii;	/* iterator. */

	for ( ii = pu->ap; ii < ZD_NUM_PORTS + pu->ap; ii++ )
	{
		( void )EnableWindow( ii->hg, fEna );
		( void )EnableWindow( ii->hg, fEna );
		( void )EnableWindow( ii->hbIn, fEna );
		( void )EnableWindow( ii->heIn, fEna );
		( void )EnableWindow( ii->heOut, fEna );
		( void )EnableWindow( ii->hbOut, fEna );

		( void )EnableWindow( ii->mon.hg, fEna );
		( void )EnableWindow( ii->mon.hsMode, fEna );
		( void )EnableWindow( ii->mon.hcMode, fEna );
		( void )EnableWindow( ii->mon.hsMask, fEna );
		( void )EnableWindow( ii->mon.heMask, fEna );
		( void )EnableWindow( ii->mon.hsComp, fEna );
		( void )EnableWindow( ii->mon.heComp, fEna );
		( void )EnableWindow( ii->mon.hbSet, fEna );
		( void )EnableWindow( ii->mon.hbStat, fEna );
		( void )EnableWindow( ii->mon.hsStat, fEna );
		( void )EnableWindow( ii->mon.heStat, fEna );
		( void )EnableWindow( ii->mon.hsData, fEna );
		( void )EnableWindow( ii->mon.heData, fEna );

		( void )EnableWindow( ii->wdt.hg, fEna );
		( void )EnableWindow( ii->wdt.hsOut, fEna );
		( void )EnableWindow( ii->wdt.heOut, fEna );
		( void )EnableWindow( ii->wdt.hbSet, fEna );
	}
}
}

/*//////////////////////////////////////////////////////////////////////
*/

static	LPCTSTR const s_apszType[]	=
{
	_T( "*00*" ),
	_T( "CPI-ZD132" ),
	_T( "CPI-ZD232" ),
	_T( "CPI-ZD332" ),
	_T( "*04*" ),
	_T( "*05*" ),
	_T( "*06*" ),
	_T( "*07*" ),

	_T( "*08*" ),
	_T( "CPI-UD132" ),
	_T( "CPI-UD232" ),
	_T( "CPI-UD332" ),
	_T( "*0C*" ),
	_T( "*0D*" ),
	_T( "*0E*" ),
	_T( "*0F*" ),

	NULL
};

static	_inline LPCTSTR str_type( UINT iT )
{
	return s_apszType[ 15 & iT ];
}

/*//////////////////////////////////////////////////////////////////////
16iQōĕ\
*/

static	void en_kf_02x( HWND he )
{
	TCHAR	sz[ 1 + 2 ];	/* value. */

	assert( NULL != he );

	switch ( GetWindowText( he, sz, LENGTH_OF( sz ) ) )
	{
	default:
		break;

	case 0:
		( void )SetWindowText( he, s_sz00 );
		break;

	case 1:
		sz[ 1 ] = sz[ 0 ];
		sz[ 0 ] = '0';
		sz[ 2 ] = 0;
		( void )SetWindowText( he, sz );
		break;

	case 2:
		/*
		// do nothing.
		*/

		break;
	}
}

/*//////////////////////////////////////////////////////////////////////
*/

DSCT( s_szF_dX_Ln,	" %d.\n" );
DSCT( s_szF_0x02X_Ln,	" 0x%02X.\n" );

static	_inline void ok_ng( LPZDX32 pu, int ri )
{
	el_putf( pu->hl,
			( 0 > ri ) ? s_szF_dX_Ln : s_szF_0x02X_Ln,
			ri
			);
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_end( LPZDX32 pu )
{
	( void )PostMessage( pu->hd, WM_CLOSE, 0, 0 );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_1st( LPZDX32 pu )
{
	BYTE	bS;	/* unit select switch. */
			/* caption. */
	TCHAR	sz[ 1 + _MAX_PATH ];
	int	ri;	/* result. */

	if ( ZXR_OK != ZdNop( pu->hv ) )
	{
		el_putf( pu->hl,
				_T( "MAY BE UD OR CONFIGURATION MODE.\n" )
				);

		return;
	}

	ri = ZdSwtGet( pu->hv, &bS );
	if ( ZXR_OK != ri )
	{
		el_putf( pu->hl,
				_T( "ZdSwtGet %d FAILED (0x%02X).\n" ),
				pu->hv->ix,
				ri
				);

		return;
	}

	( void )_stprintf_s(
			sz,
			LENGTH_OF( sz ),
			_T( "%hs sw:%02X %08lX-%08lX (%d)" ),
			pu->hv->ni,
			bS,
			pu->hv->sh,
			pu->hv->sl,
			pu->hv->ix
			);

	( void )SetWindowText( pu->hd, sz );

	ena_all( pu, TRUE );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_nop( LPZDX32 pu )
{
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdNop %d ..." ), pu->hv->ix );
	ri = ZdNop( pu->hv );
	ok_ng( pu, ri );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_err( LPZDX32 pu )
{
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdErrGet %d ..." ), pu->hv->ix );
	ri = ZdErrGet( pu->hv );
	el_putf( pu->hl, s_szF_0x02X_Ln, ri );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_name( LPZDX32 pu )
{
			/* unit name. */
	char	sz[ 1 + XB_CCH_MAX_NI ];
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdDevGet %d ..." ), pu->hv->ix );
	ri = ZdDevGet( pu->hv, sz, LENGTH_OF( sz ) );
	ok_ng( pu, ri );
	if ( ZXR_OK != ri )
	{
		return;
	}

	el_putf( pu->hl, _T( "Unit name: %hs.\n" ), sz );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_ver( LPZDX32 pu )
{
			/* version. */
	char	sz[ 1 + 4 ];
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdVerGet %d ..." ), pu->hv->ix );
	ri = ZdVerGet( pu->hv, sz, LENGTH_OF( sz ) );
	ok_ng( pu, ri );
	if ( ZXR_OK != ri )
	{
		return;
	}

	el_putf( pu->hl, _T( "Firmware version: %hs.\n" ), sz );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_type( LPZDX32 pu )
{
	BYTE	bV;	/* type. */
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdTypeGet %d ..." ), pu->hv->ix );
	ri = ZdTypeGet( pu->hv, &bV );
	ok_ng( pu, ri );
	if ( ZXR_OK != ri )
	{
		return;
	}

	el_putf( pu->hl,
			_T( "Unit type: %d %s.\n" ),
			bV,
			str_type( bV )
			);
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_swt( LPZDX32 pu )
{
	BYTE	bV;	/* type. */
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdSwtGet %d ..." ), pu->hv->ix );
	ri = ZdSwtGet( pu->hv, &bV );
	ok_ng( pu, ri );
	if ( ZXR_OK != ri )
	{
		return;
	}

	el_putf( pu->hl,
			_T( "Unit select switch value: 0x%02X.\n" ),
			bV
			);
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_atdb( LPZDX32 pu )
{
	BYTE	db;	/* -[dBm] */
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdAtdb %d ..." ), pu->hv->ix );
	ri = ZdAtdb( pu->hv, &db );
	ok_ng( pu, ri );
	if ( ZXR_OK != ri )
	{
		return;
	}

	el_putf( pu->hl, _T( "atdb: -%ddBm.\n" ), db );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_filt_set( LPZDX32 pu )
{
	BYTE	iFreq;	/* interval. */
	BYTE	cStep;	/* steps. */
	int	ri;	/* result. */

	iFreq = ( BYTE )SendMessage(
			pu->comn.filt.hcFreq,
			CB_GETCURSEL,
			0,
			0
			);

	cStep = ( BYTE )SendMessage(
			pu->comn.filt.hcStep,
			CB_GETCURSEL,
			0,
			0
			);

	el_putf( pu->hl, 
			_T( "ZdFiltSet %d %d %d ..." ),
			pu->hv->ix,
			iFreq,
			cStep
			);
	ri = ZdFiltSet( pu->hv, iFreq, cStep );
	ok_ng( pu, ri );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_wdt_set( LPZDX32 pu )
{
	BYTE	iTmo;	/* interval. */
	int	ri;	/* result. */

	iTmo = ( BYTE )SendMessage(
			pu->comn.wdt.hcTmo,
			CB_GETCURSEL,
			0,
			0
			);

	el_putf( pu->hl, _T( "ZdWdtSet %d %d ..." ), pu->hv->ix, iTmo );
	ri = ZdWdtSet( pu->hv, iTmo );
	ok_ng( pu, ri );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_wdt_rst( LPZDX32 pu )
{
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdWdtRst %d ..." ), pu->hv->ix );
	ri = ZdWdtRst( pu->hv );
	ok_ng( pu, ri );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_wdt_stat( LPZDX32 pu )
{
	BYTE	bV;	/* interval. */
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdWdtStatGet %d ..." ), pu->hv->ix );
	ri = ZdWdtStatGet( pu->hv, &bV );
	ok_ng( pu, ri );
	if ( ZXR_OK != ri )
	{
		return;
	}

	el_putf( pu->hl, _T( "WDT status: %d.\n" ), bV );

{
			/* status. */
	TCHAR	sz[ 1 + 7 ];

	( void )_stprintf_s( sz, LENGTH_OF( sz ), s_szF_d, bV );
	( void )SetWindowText( pu->comn.wdt.heStat, sz );
}
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_wdt_clr( LPZDX32 pu )
{
	int	ri;	/* result. */

	el_putf( pu->hl, _T( "ZdWdtClr %d ..." ), pu->hv->ix );
	ri = ZdWdtClr( pu->hv );
	ok_ng( pu, ri );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_in( LPZDX32 pu, UINT iP )
{
	BYTE	bV;	/* input value. */
	int	ri;	/* result. */

	assert( ZD_NUM_PORTS > iP );

	el_putf( pu->hl, _T( "ZdPortIn %d %d ..." ), pu->hv->ix, iP );
	ri = ZdPortIn( pu->hv, ( BYTE )iP, &bV );
	ok_ng( pu, ri );
	if ( ZXR_OK != ri )
	{
		return;
	}

	el_putf( pu->hl, _T( "Input value: 0x%02X.\n" ), bV );

{
	TCHAR	sz[ 1 + 2 ];

	( void )_stprintf_s( sz, LENGTH_OF( sz ), s_szF_02X, bV );
	( void )SetWindowText( pu->ap[ iP ].heIn, sz );
}
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_out( LPZDX32 pu, UINT iP )
{
			/* output value. */
	TCHAR	sz[ 8 ];
	BYTE	bV;	/* output value. */
	int	ri;	/* result. */

	assert( ZD_NUM_PORTS > iP );

	if ( 1 > GetWindowText(
			pu->ap[ iP ].heOut,
			sz,
			LENGTH_OF( sz )
			) )
	{
		el_putf( pu->hl,
				_T( "\
ZdPortOut %d %d: GetWindowText heOut FAILED.\r\n\
Windows error %ld\r\n\
"						),
				pu->hv->ix,
				iP,
				GetLastError()
				);

		return;
	}

	bV = ( BYTE )_tcstoul( sz, NULL, 0x10 );

	el_putf( pu->hl, 
			_T( "ZdPortOut %d %d 0x%02X ..." ),
			pu->hv->ix,
			iP,
			bV
			);
	ri = ZdPortOut( pu->hv, ( BYTE )iP, bV );
	ok_ng( pu, ri );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_mon_set( LPZDX32 pu, UINT iP )
{
	LPPORT	pP;	/* port control. */
	BYTE	iMode;	/* mode. */
	BYTE	bm;	/* bitmap. */
	BYTE	bComp;	/* compare data. */
			/* hexadecimal value. */
	TCHAR	sz[ 8 ];
	int	ri;	/* result. */

	assert( ZD_NUM_PORTS > iP );

	pP = iP + pu->ap;

	iMode = ( BYTE )SendMessage(
			pP->mon.hcMode,
			CB_GETCURSEL,
			0,
			0
			);

	if ( 1 > GetWindowText( pP->mon.heMask, sz, LENGTH_OF( sz ) ) )
	{
		el_putf( pu->hl,
				_T( "\
ZdMonSet %d %d: GetWindowText heMask FAILED.\r\n\
Windows error %ld.\r\n\
"						),
				pu->hv->ix,
				iP,
				GetLastError()
				);

		return;
	}

	bm = ( BYTE )_tcstoul( sz, NULL, 0x10 );

	if ( 1 > GetWindowText( pP->mon.heComp, sz, LENGTH_OF( sz ) ) )
	{
		el_putf( pu->hl,
				_T( "\
ZdMonSet %d %d: GetWindowText heComp FAILED.\r\n\
Windows error %ld.\r\n\
"						),
				pu->hv->ix,
				iP,
				GetLastError()
				);

		return;
	}

	bComp = ( BYTE )_tcstoul( sz, NULL, 0x10 );

	el_putf( pu->hl, 
			_T( "ZdMonSet %d %d %d 0x%02X 0x%02X ..." ),
			pu->hv->ix,
			iP,
			iMode,
			bm,
			bComp
			);
	ri = ZdMonSet( pu->hv, ( BYTE )iP, iMode, bm, bComp );
	ok_ng( pu, ri );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_mon_stat( LPZDX32 pu, UINT iP )
{
	LPPORT	pP;	/* port control. */
	BYTE	bS;	/* status. */
	BYTE	bD;	/* data. */
			/* hexadecimal value. */
	TCHAR	sz[ 1 + 2 ];
	int	ri;	/* result. */

	assert( NULL != pu );
	assert( ZD_NUM_PORTS > iP );

	pP = iP + pu->ap;

	el_putf( pu->hl, 
			_T( "ZdMonStatGet %d %d ..." ),
			pu->hv->ix,
			iP
			);
	ri = ZdMonStatGet( pu->hv, ( BYTE )iP, &bS, &bD );
	ok_ng( pu, ri );
	if ( ZXR_OK != ri )
	{
		return;
	}

	el_putf( pu->hl, 
			_T( "Status: 0x%02X, Data: 0x%02X.\n" ),
			bS,
			bD
			);

	( void )_stprintf_s( sz, LENGTH_OF( sz ), s_szF_02X, bS );
	( void )SetWindowText( pP->mon.heStat, sz );

	( void )_stprintf_s( sz, LENGTH_OF( sz ), s_szF_02X, bD );
	( void )SetWindowText( pP->mon.heData, sz );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cmnd_wdt_out( LPZDX32 pu, UINT iP )
{
			/* output value. */
	TCHAR	sz[ 8 ];
	BYTE	bV;	/* output value. */
	int	ri;	/* result. */

	assert( ZD_NUM_PORTS > iP );

	if ( 1 > GetWindowText(
			pu->ap[ iP ].wdt.heOut,
			sz,
			LENGTH_OF( sz )
			) )
	{
		el_putf( pu->hl,
				_T( "\
ZdWdtOutSet %d %d: GetWindowText wdt.heOut FAILED.\r\n\
Windows error %ld.\r\n\
"						),
				pu->hv->ix,
				iP,
				GetLastError()
				);

		return;
	}

	bV = ( BYTE )_tcstoul( sz, NULL, 0x10 );

	el_putf( pu->hl, 
			_T( "ZdWdtOutSet %d %d 0x%02X ..." ),
			pu->hv->ix,
			iP,
			bV
			);
	ri = ZdWdtOutSet( pu->hv, ( BYTE )iP, bV );
	ok_ng( pu, ri );
}
/*//////////////////////////////////////////////////////////////////////
*/

static	BOOL on_cmnd( HWND hd, WPARAM wP, LPARAM lP )
{
	LPZDX32	pu;	/* unit information. */

	/*
	// obtain my device information.
	// 擾`̂}kPۂ
	*/

	pu = ( LPZDX32 )WLGet( hd, WL_USERDATA );
	assert( NULL != pu );

	switch ( wP )
	{
	default:
		return FALSE;		/* message not handled. */

	case MAKELONG( IDOK, BN_CLICKED ):
		cmnd_end( pu );
		break;

	case MAKELONG( IDCANCEL, BN_CLICKED ):
		cmnd_end( pu );
		break;

	case MAKELONG( IDRETRY, BN_CLICKED ):
		cmnd_1st( pu );
		cur_norm();
		break;

	case MAKELONG( IDC_BUTN_LOG_CLEAR, BN_CLICKED ):
		el_clr( pu->hl );
		break;

	case MAKELONG( IDC_BUTN_NOP, BN_CLICKED ):
		cmnd_nop( pu );
		break;

	case MAKELONG( IDC_BUTN_GET_LAST_ERROR, BN_CLICKED ):
		cmnd_err( pu );
		break;

	case MAKELONG( IDC_BUTN_GET_VERSION, BN_CLICKED ):
		cmnd_ver( pu );
		break;

	case MAKELONG( IDC_BUTN_GET_UNIT_NAME, BN_CLICKED ):
		cmnd_name( pu );
		break;

	case MAKELONG( IDC_BUTN_GET_UNIT_TYPE, BN_CLICKED ):
		cmnd_type( pu );
		break;

	case MAKELONG( IDC_BUTN_GET_SWITCH_VALUE, BN_CLICKED ):
		cmnd_swt( pu );
		break;

	case MAKELONG( IDC_BUTN_ATDB, BN_CLICKED ):
		cmnd_atdb( pu );
		break;

	case MAKELONG( IDC_BUTN_FILT_SET, BN_CLICKED ):
		cmnd_filt_set( pu );
		break;

	case MAKELONG( IDC_BUTN_WDT_SET, BN_CLICKED ):
		cmnd_wdt_set( pu );
		break;

	case MAKELONG( IDC_BUTN_WDT_RESET, BN_CLICKED ):
		cmnd_wdt_rst( pu );
		break;

	case MAKELONG( IDC_BUTN_WDT_STAT_GET, BN_CLICKED ):
		cmnd_wdt_stat( pu );
		break;

	case MAKELONG( IDC_BUTN_WDT_CLEAR, BN_CLICKED ):
		cmnd_wdt_clr( pu );
		break;

	case MAKELONG( IDC_BUTN_IN_A, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_IN_B, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_IN_C, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_IN_D, BN_CLICKED ):
		cmnd_in( pu, LOWORD( wP ) - IDC_BUTN_IN_A );
		break;

	case MAKELONG( IDC_BUTN_OUT_A, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_OUT_B, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_OUT_C, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_OUT_D, BN_CLICKED ):
		cmnd_out( pu, LOWORD( wP ) - IDC_BUTN_OUT_A );
		break;

	case MAKELONG( IDC_BUTN_MON_SET_A, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_MON_SET_B, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_MON_SET_C, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_MON_SET_D, BN_CLICKED ):
		cmnd_mon_set( pu, LOWORD( wP ) - IDC_BUTN_MON_SET_A );
		break;

	case MAKELONG( IDC_BUTN_MON_STAT_A, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_MON_STAT_B, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_MON_STAT_C, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_MON_STAT_D, BN_CLICKED ):
		cmnd_mon_stat( pu, LOWORD( wP ) - IDC_BUTN_MON_STAT_A );
		break;

	case MAKELONG( IDC_BUTN_WDT_OUT_A, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_WDT_OUT_B, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_WDT_OUT_C, BN_CLICKED ):
	case MAKELONG( IDC_BUTN_WDT_OUT_D, BN_CLICKED ):
		cmnd_wdt_out( pu, LOWORD( wP ) - IDC_BUTN_WDT_OUT_A );
		break;

	case MAKELONG( IDC_EDIT_OUT_A, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_OUT_B, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_OUT_C, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_OUT_D, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_MON_MASK_A, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_MON_MASK_B, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_MON_MASK_C, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_MON_MASK_D, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_MON_COMP_A, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_MON_COMP_B, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_MON_COMP_C, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_MON_COMP_D, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_WDT_OUT_A, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_WDT_OUT_B, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_WDT_OUT_C, EN_KILLFOCUS ):
	case MAKELONG( IDC_EDIT_WDT_OUT_D, EN_KILLFOCUS ):
		en_kf_02x( ( HWND )lP );
		break;
	}

	return TRUE;			/* message handled. */
}

/*//////////////////////////////////////////////////////////////////////
*/

static	_inline void cb_add_strs( HWND hc, LPCTSTR const FAR* ppsz )
{
	for ( ; NULL != *ppsz; ppsz++ )
	{
		( void )SendMessage(
				hc,
				CB_ADDSTRING,
				0,
				( LPARAM )*ppsz
				);
	}
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void cb_add_deci( HWND hc, UINT iMin, UINT iMax )
{
	TCHAR	sz[ 8 ];

	for ( ; iMin <= iMax; iMin++ )
	{
		( void )_itot_s( iMin, sz, LENGTH_OF( sz ), 10 );

		( void )SendMessage(
				hc,
				CB_ADDSTRING,
				0,
				( LPARAM )sz
				);
	}
}

/*//////////////////////////////////////////////////////////////////////
*/

static	_inline void cb_ini_strs( HWND hc, LPCTSTR const FAR* ppsz, UINT i0 )
{
	( void )SendMessage( hc, CB_RESETCONTENT, 0, 0 );

	cb_add_strs( hc, ppsz );

	( void )SendMessage( hc, CB_SETCURSEL, i0, 0 );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	_inline void cb_ini_deci( HWND hc, UINT iMin, UINT iMax, UINT i0 )
{
	( void )SendMessage( hc, CB_RESETCONTENT, 0, 0 );

	cb_add_deci( hc, iMin, iMax );

	( void )SendMessage( hc, CB_SETCURSEL, i0, 0 );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void filt_freq_init( HWND hc )
{
static	LPCTSTR const s_apsz[]	=
{
	_T( "STOP" ),
	_T( "1260uS" ),
	_T( "2520uS" ),
	_T( "5040uS" ),
	_T( "10080uS" ),
	_T( "20160uS" ),
	NULL
};

	cb_ini_strs( hc, s_apsz, 0 );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void filt_mat_init( HWND hc )
{
	cb_ini_deci( hc, 0, ( BYTE ) ~ 0, 0 );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void wdt_tmo_init( HWND hc )
{
static	LPCTSTR const s_apsz[]	=
{
	_T( "STOP" ),
	_T( "126mS" ),
	_T( "252mS" ),
	_T( "504mS" ),
	_T( "1008mS" ),
	_T( "2016mS" ),
	NULL
};

	( void )SendMessage( hc, CB_RESETCONTENT, 0, 0 );

	cb_add_strs( hc, s_apsz );

{
	TCHAR	sz[ 8 ];
	LPTSTR	psz;
	UINT	bV;

	for ( bV = LENGTH_OF( s_apsz ) - 1; bV <= ( BYTE ) ~ 0; bV++ )
	{
		( void )_itot_s( bV, sz, LENGTH_OF( sz ), 10 );

		for ( psz = sz; *psz; psz++ )
		{
			;
		}
		*psz++ = 'S';
		*psz = 0;

		( void )SendMessage(
				hc,
				CB_ADDSTRING,
				0,
				( LPARAM )sz
				);
	}
}

	( void )SendMessage( hc, CB_SETCURSEL, 0, 0 );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void mon_mode_init( HWND hc )
{
static	LPCTSTR s_apsz[]	=
{
	_T( "NON" ),
	_T( "ALT" ),
	_T( "AND" ),
	_T( "OR" ),
	NULL
};

	cb_ini_strs( hc, s_apsz, ZD_MON_ALT );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	void port_init( LPZDX32 pu, UINT iP )
{
	LPPORT	pP;	/* port control. */

	assert( NULL != pu->hd );
	assert( ZD_NUM_PORTS > iP );

	pP = iP + pu->ap;
	assert( NULL != pP );

	pP->hg = GetDlgItem( pu->hd, IDC_GRBX_PORT_A + iP );

	pP->hbIn = GetDlgItem( pu->hd, IDC_BUTN_IN_A + iP );

	pP->heIn = GetDlgItem( pu->hd, IDC_EDIT_IN_A + iP );
	( void )SendMessage( pP->heIn, EM_LIMITTEXT, 2, 0 );

	pP->heOut = GetDlgItem( pu->hd, IDC_EDIT_OUT_A + iP );
	( void )e16( pP->heOut, 2 );
	( void )SetWindowText( pP->heOut, s_sz00 );

	pP->hbOut = GetDlgItem( pu->hd, IDC_BUTN_OUT_A + iP );

	pP->mon.hg = GetDlgItem( pu->hd, IDC_GRBX_MON_A + iP );

	pP->mon.hsMode = GetDlgItem( pu->hd, IDC_STAT_MON_MODE_A + iP );

	pP->mon.hcMode = GetDlgItem( pu->hd, IDC_CBBX_MON_MODE_A + iP );
	mon_mode_init( pP->mon.hcMode );

	pP->mon.hsMask = GetDlgItem( pu->hd, IDC_STAT_MON_MASK_A + iP );

	pP->mon.heMask = GetDlgItem( pu->hd, IDC_EDIT_MON_MASK_A + iP );
	( void )e16( pP->mon.heMask, 2 );
	( void )SetWindowText( pP->mon.heMask, s_szFF );

	pP->mon.hsComp = GetDlgItem( pu->hd, IDC_STAT_MON_COMP_A + iP );

	pP->mon.heComp = GetDlgItem( pu->hd, IDC_EDIT_MON_COMP_A + iP );
	( void )e16( pP->mon.heComp, 2 );
	( void )SetWindowText( pP->mon.heComp, s_sz00 );

	pP->mon.hbSet = GetDlgItem( pu->hd, IDC_BUTN_MON_SET_A + iP );

	pP->mon.hbStat = GetDlgItem( pu->hd, IDC_BUTN_MON_STAT_A + iP );

	pP->mon.hsStat = GetDlgItem( pu->hd, IDC_STAT_MON_STAT_A + iP );

	pP->mon.heStat = GetDlgItem( pu->hd, IDC_EDIT_MON_STAT_A + iP );
	( void )e16( pP->mon.heStat, 2 );

	pP->mon.hsData = GetDlgItem( pu->hd, IDC_STAT_MON_DATA_A + iP );

	pP->mon.heData = GetDlgItem( pu->hd, IDC_EDIT_MON_DATA_A + iP );
	( void )e16( pP->mon.heData, 2 );

	pP->wdt.hg = GetDlgItem( pu->hd, IDC_GRBX_WDT_A + iP );

	pP->wdt.hsOut = GetDlgItem( pu->hd, IDC_STAT_WDT_OUT_A + iP );

	pP->wdt.heOut = GetDlgItem( pu->hd, IDC_EDIT_WDT_OUT_A + iP );
	( void )e16( pP->wdt.heOut, 2 );
	( void )SetWindowText( pP->wdt.heOut, s_sz00 );

	pP->wdt.hbSet = GetDlgItem( pu->hd, IDC_BUTN_WDT_OUT_A + iP );
}

/*//////////////////////////////////////////////////////////////////////
*/

static	BOOL on_init( HWND hd, LPZDX32 pu )
{
	assert( NULL != pu );
	assert( NULL != hd );

	pu->hd = hd;

	/*
	// remember device information.
	*/

	( void )WLSet( hd, WL_USERDATA, ( LONG_PTR )pu );

	/*
	//
	*/

	pu->hbNop = GetDlgItem( hd, IDC_BUTN_NOP );

	pu->hbErr = GetDlgItem( hd, IDC_BUTN_GET_LAST_ERROR );

	pu->hbName = GetDlgItem( hd, IDC_BUTN_GET_UNIT_NAME );

	pu->hbVer = GetDlgItem( hd, IDC_BUTN_GET_VERSION );

	pu->hbType = GetDlgItem( hd, IDC_BUTN_GET_UNIT_TYPE );

	pu->hbSwt = GetDlgItem( hd, IDC_BUTN_GET_SWITCH_VALUE );

	pu->hbAtdb = GetDlgItem( hd, IDC_BUTN_ATDB );

	pu->comn.hg = GetDlgItem( hd, IDC_GRBX_COMMON );

	pu->comn.filt.hg = GetDlgItem( hd, IDC_GRBX_FILT );

	pu->comn.filt.hsFreq = GetDlgItem( hd, IDC_STAT_FILT_FREQ );

	pu->comn.filt.hcFreq = GetDlgItem( hd, IDC_CBBX_FILT_FREQ );
	filt_freq_init( pu->comn.filt.hcFreq );

	pu->comn.filt.hsStep = GetDlgItem( hd, IDC_STAT_FILT_STEP );

	pu->comn.filt.hcStep = GetDlgItem( hd, IDC_CBBX_FILT_STEP );
	filt_mat_init( pu->comn.filt.hcStep );

	pu->comn.filt.hbSet = GetDlgItem( hd, IDC_BUTN_FILT_SET );

	pu->comn.wdt.hg = GetDlgItem( hd, IDC_GRBX_WDT );

	pu->comn.wdt.hsTmo = GetDlgItem( hd, IDC_STAT_WDT_TMO );

	pu->comn.wdt.hcTmo = GetDlgItem( hd, IDC_CBBX_WDT_TMO );
	wdt_tmo_init( pu->comn.wdt.hcTmo );

	pu->comn.wdt.hbSet = GetDlgItem( hd, IDC_BUTN_WDT_SET );

	pu->comn.wdt.hbRst = GetDlgItem( hd, IDC_BUTN_WDT_RESET );

	pu->comn.wdt.hbStat = GetDlgItem( hd, IDC_BUTN_WDT_STAT_GET );

	pu->comn.wdt.heStat = GetDlgItem( hd, IDC_EDIT_WDT_STAT );
	( void )SendMessage( pu->comn.wdt.heStat, EM_LIMITTEXT, 2, 0 );

	pu->comn.wdt.hbClr = GetDlgItem( hd, IDC_BUTN_WDT_CLEAR );


{
	UINT	iP;	/* port iterator. */

	for ( iP = 0; iP < ZD_NUM_PORTS; iP++ )
	{
		port_init( pu, iP );		/* port controls. */
	}
}

	pu->hl = el_cat( GetDlgItem( hd, IDC_EDIT_LOG ), 12 );

	/*
	// at first.
	*/

	( void )PostMessage(
			pu->hd,
			WM_COMMAND,
			MAKELONG( IDRETRY, BN_CLICKED ),
			0
			);

	return TRUE;			/* set default focus. */
}
