//<summary>
// SPN.cs
// get friendly serial port names.
// Copyright (C) CPI Technologies, Inc.
//</summary>

using System;
using System.IO.Ports;
using System.Runtime.InteropServices;

using Win32API;

namespace System.IO.Ports
{
    public class SPN
    {
        private const string scCom = "COM";

        //<summary>
        // get friendly serial port names.
        //  ret friendly serial port names.
        //</summary>

        public static string[] GetFriendlyPortNames()
        {
            const int REG_SZ = 1;

            // foCXZbg擾܂B

            Guid guid = new Guid(SetupAPI.GUID_DEVCLASS_PORTS);
            IntPtr hvi = SetupAPI.SetupDiGetClassDevs(
                    ref guid,
                    0,
                    IntPtr.Zero,
                    SetupAPI.DIGCF_PRESENT
                    );
            if (IntPtr.Zero.Equals(hvi))
            {
                // ʐM|[gfoCX܂B

                return null;
            }

            // VA|[gv܂B

            int cb = 2048;
            IntPtr pb = Marshal.AllocHGlobal(cb);
            int dT;
            int cbReq;
            SetupAPI.SP_DEVINFO_DATA did = new SetupAPI.SP_DEVINFO_DATA();

            int nV = 0;
            int ii;
            for (ii = 0; ; ii++)
            {
                if (! SetupAPI.SetupDiEnumDeviceInfo(hvi, ii, did))
                {
                    break;
                }

                if (! SetupAPI.SetupDiGetDeviceRegistryProperty(
                        hvi,
                        did,
                        SetupAPI.SPDRP_FRIENDLYNAME,
                        out dT,
                        pb,
                        cb,
                        out cbReq
                        ))
                {
                    continue;
                }

                // 擾ꂽ͕^łH

                if (REG_SZ != dT)
                {
                    // ^ł͂܂B
                    // ʐM|[gł͂ȂƔf܂B
                    // EEE

                    continue;
                }

                // ʐM|[gł傤H
                //    "ʐM|[g (COM1)"
                //    "USB Serial Port (COM3)"
                //    Ɋ܂܂"COM"T܂B
                // TODO
                //    "ECP v^ |[g (LPT1)"
                //    Ȃǂ"COM"܂܂ĂAEgłB

                string sc = Marshal.PtrToStringAuto(pb);
                if (! sc.Contains(scCom))
                {
                    // "COM?" ̌`ł͂܂B
                    // EEE

                    continue;
                }

                // VA|[głB

                nV++;
            }

            // friendly serial port name擾܂B

            string[] asc = new string[nV];

            nV = 0;
            for (ii = 0; ; ii++)
            {
                if (! SetupAPI.SetupDiEnumDeviceInfo(hvi, ii, did))
                {
                    break;
                }

                if (! SetupAPI.SetupDiGetDeviceRegistryProperty(
                        hvi,
                        did,
                        SetupAPI.SPDRP_FRIENDLYNAME,
                        out dT,
                        pb,
                        cb,
                        out cbReq
                        ))
                {
                    continue;
                }

                // 擾ꂽ͕^łH

                if (REG_SZ != dT)
                {
                    // ^ł͂܂B
                    // ʐM|[gł͂ȂƔf܂B
                    // EEE

                    continue;
                }

                // ʐM|[gł傤H
                //    "ʐM|[g (COM1)"
                //    "USB Serial Port (COM3)"
                //    Ɋ܂܂"COM"T܂B
                // TODO
                //    "ECP v^ |[g (LPT1)"
                //    Ȃǂ"COM"܂܂ĂAEgłB

                string sc = Marshal.PtrToStringAuto(pb);
                if (! sc.Contains(scCom))
                {
                    // "COM?" ̌`ł͂܂B
                    // EEE

                    continue;
                }

                // friendly serial port nameݒ肵܂B

                asc[nV++] = sc;
            }

            Marshal.FreeHGlobal(pb);

            SetupAPI.SetupDiDestroyDeviceInfoList(hvi);

            return asc;
        }

        //<summary>
        // obtain serial port name from friendly serial port name.
        //  in  sc      friendly serial port name.
        //  ret null    failed.
        //      else    serial port name.
        //</summary>

        public static string ConvertToPortName(string sc)
        {
            if (null == sc)
            {
                return null;
            }

            int ix = sc.IndexOf(scCom);
            if (0 > ix)
            {
                return null;
            }

            string scNum = "";
            for (ix += 3; ix < sc.Length; ix++)
            {
                char ch = sc[ix];
                if (! Char.IsDigit(ch))
                {
                    break;
                }

                scNum += ch;
            }

            if (1 > scNum.Length)
            {
                return null;
            }

            return scCom + scNum;
        }
    }
}
