//<summary>
// SetupAPI.cs
// setup API interface.
// Copyright (C) CPI Technologies, Inc.
//</summary>

using System;
using System.Runtime.InteropServices;

namespace Win32API
{
    public class SetupAPI
    {
        public const string GUID_DEVINTERFACE_DISK      = "53f56307-b6bf-11d0-94f2-00a0c91efb8b";
        public const string GUID_DEVINTERFACE_VOLUME    = "53f5630d-b6bf-11d0-94f2-00a0c91efb8b";
        public const string GUID_DEVCLASS_PORTS         = "4d36e978-e325-11ce-bfc1-08002be10318";

        public const int DIGCF_DEFAULT          = 0x00000001;   // only valid with DIGCF_DEVICEINTERFACE
        public const int DIGCF_PRESENT          = 0x00000002;
        public const int DIGCF_ALLCLASSES       = 0x00000004;
        public const int DIGCF_PROFILE          = 0x00000008;
        public const int DIGCF_DEVICEINTERFACE  = 0x00000010;

        public const int SPDRP_DEVICEDESC       = 0x00000000;   // DeviceDesc  = R/W
        public const int SPDRP_HARDWAREID       = 0x00000001;   // HardwareID  = R/W
        public const int SPDRP_COMPATIBLEIDS    = 0x00000002;   // CompatibleIDs  = R/W
        public const int SPDRP_NTDEVICEPATHS    = 0x00000003;   // Unsupported, DO NOT USE
        public const int SPDRP_SERVICE          = 0x00000004;   // Service  = R/W
        public const int SPDRP_CONFIGURATION    = 0x00000005;   // Configuration  = R
        public const int SPDRP_CONFIGURATIONVECTOR
                                                = 0x00000006;   // ConfigurationVector  = R
        public const int SPDRP_CLASS            = 0x00000007;   // Class  = R--tied to ClassGUID
        public const int SPDRP_CLASSGUID        = 0x00000008;   // ClassGUID  = R/W
        public const int SPDRP_DRIVER           = 0x00000009;   // Driver  = R/W
        public const int SPDRP_CONFIGFLAGS      = 0x0000000A;   // ConfigFlags  = R/W
        public const int SPDRP_MFG              = 0x0000000B;   // Mfg  = R/W
        public const int SPDRP_FRIENDLYNAME     = 0x0000000C;   // FriendlyName  = R/W
        public const int SPDRP_LOCATION_INFORMATION
                                                = 0x0000000D;   // LocationInformation  = R/W
        public const int SPDRP_PHYSICAL_DEVICE_OBJECT_NAME
                                                = 0x0000000E;   // PhysicalDeviceObjectName  = R
        public const int SPDRP_CAPABILITIES     = 0x0000000F;   // Capabilities  = R
        public const int SPDRP_UI_NUMBER        = 0x00000010;   // UiNumber  = R
        public const int SPDRP_UPPERFILTERS     = 0x00000011;   // UpperFilters  = R/W
        public const int SPDRP_LOWERFILTERS     = 0x00000012;   // LowerFilters  = R/W
        public const int SPDRP_MAXIMUM_PROPERTY = 0x00000013;   // Upper bound on ordinals

        [StructLayout(LayoutKind.Sequential)]
        public class SP_DEVINFO_DATA
        {
            public int cbSize = Marshal.SizeOf( typeof( SP_DEVINFO_DATA ) );
            public Guid classGuid = Guid.Empty; // temp
            public int devInst = 0; // dumy
            public int reserved = 0;
        }

        [StructLayout(LayoutKind.Sequential, Pack = 2)]
        public struct SP_DEVICE_INTERFACE_DETAIL_DATA
        {
            public int cbSize;
            public short devicePath;
        }

        [StructLayout(LayoutKind.Sequential)]
        public class SP_DEVICE_INTERFACE_DATA
        {
            public int cbSize = Marshal.SizeOf(
                    typeof( SP_DEVICE_INTERFACE_DATA )
                    );
            public Guid interfaceClassGuid = Guid.Empty; // temp
            public int flags = 0;
            public int reserved = 0;
        }

        [DllImport("setupapi.dll")]
        public static extern IntPtr SetupDiGetClassDevs(
                ref Guid classGuid,
                int enumerator,
                IntPtr hwndParent,
                int flags
                );

        [DllImport("setupapi.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool SetupDiEnumDeviceInfo(
                IntPtr deviceInfoSet,
                int memberIndex,
                SP_DEVINFO_DATA deviceInfoData
                );

        [DllImport("setupapi.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool SetupDiEnumDeviceInterfaces(
                IntPtr deviceInfoSet,
                SP_DEVINFO_DATA deviceInfoData,
                ref Guid interfaceClassGuid,
                int memberIndex,
                SP_DEVICE_INTERFACE_DATA deviceInterfaceData
                );

        [DllImport("setupapi.dll")]
        public static extern bool SetupDiOpenDeviceInfo(
                IntPtr deviceInfoSet,
                string deviceInstanceId,
                IntPtr hwndParent,
                int openFlags,
                SP_DEVINFO_DATA deviceInfoData
                );

        [DllImport("setupapi.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool SetupDiGetDeviceInterfaceDetail(
                IntPtr deviceInfoSet,
                SP_DEVICE_INTERFACE_DATA deviceInterfaceData,
                IntPtr deviceInterfaceDetailData,
                int deviceInterfaceDetailDataSize,
                ref int requiredSize,
                SP_DEVINFO_DATA deviceInfoData
                );

        [DllImport("setupapi.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool SetupDiGetDeviceRegistryProperty(
                IntPtr deviceInfoSet,
                SP_DEVINFO_DATA deviceInfoData,
                int property,
                out int propertyRegDataType,
                IntPtr propertyBuffer,
                int propertyBufferSize,
                out int requiredSize
                );

        [DllImport("setupapi.dll")]
        public static extern uint SetupDiDestroyDeviceInfoList(
                IntPtr deviceInfoSet
                );
    }
}
