using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO.Ports;
using System.IO;
using System.Threading;

namespace ns_zdx32
{
    public class XBee
    {
        // device type.
        public const byte dtCoordinator = 0;            // R[fBl[^
        public const byte dtCoord = dtCoordinator;
        public const byte dtRouter = 1;                 // [^
        public const byte dtEndDevice = 2;              // GhEfoCX
        public const byte dtDevice = dtEndDevice;
        public enum dt : byte
        {
            Coordinator = 0,            // R[fBl[^
            Coord = Coordinator,
            Router = 1,                 // [^
            EndDevice = 2,              // GhEfoCX
            Device = EndDevice,
            _top
        }

        // modem status.
        public const byte mstHwReset = 0x00;            // n[hEFAZbg
        public const byte mstWdtReset = 0x01;           // WDTZbg
                                                        // lbg[N֐ڑ
        public const byte mstJoinedNetwork = 0x02;      // ([^ƃGhfoCX)
        public const byte mstDisassociated = 0x03;      // ڑ
        public const byte mstCoordStarted = 0x06;	    // R[fBl[^Jn
        public const byte mstSecurityKeyUpd = 0x07;     // lbg[NEZLeBEL[XV
                                                        // d𒴉߁iPRO S2B̂݁j
        public const byte mstVoltageLimitExceeded = 0x0d;
        public const byte mstConfigChanged = 0x11;      // ڑɃf\ύXꂽ
        public const byte mstStackError = 0x80;         // ȏX^bNG[
        public enum mst : byte
        {
            HwReset = 0x00,             // n[hEFAZbg
            WdtReset = 0x01,            // WDTZbg
                                        // lbg[N֐ڑ
            JoinedNetwork = 0x02,       // ([^ƃGhfoCX)
            Disassociated = 0x03,       // ڑ
            CoordStarted = 0x06,	    // R[fBl[^Jn
            SecurityKeyUpd = 0x07,      // lbg[NEZLeBEL[XV
                                        // d𒴉߁iPRO S2B̂݁j
            VoltageLimitExceeded = 0x0d,
            ConfigChanged = 0x11,       // ڑɃf\ύXꂽ
            StackError = 0x80,          // ȏX^bNG[
            _top
        }

        // API frame

        // character constant.
        public const byte chDelim = 0x7e;       // f~^
        public const byte chEscape = 0x7d;      // GXP[v
        public const byte chXon = 0x11;         // XON
        public const byte chXoff = 0x13;        // XOFF
        public const byte xorEscape = 0x20;     // XOR܂

        // frame type.
        public const byte ftTxAt = 0x08;                // ATR}hv
        public const byte ftTxZb = 0x10;                // ZigBeeMv
        public const byte ftTxRat = 0x17;               // uATR}hv
        public const byte ftRxAt = ( 0x80 | ftTxAt );   // ATR}h
        public const byte ftRxMst = 0x8a;               // fXe[^X
        public const byte ftRxZbSt = 0x8b;              // ZigBeeMXe[^X
        public const byte ftRxZb = ( 0x80 | ftTxZb );   // ZigBeeMpPbg
        public const byte ftRxRat = ( 0x80 | ftTxRat ); // uATR}h
        public enum ft : byte
        {
            TxAt = 0x08,                // ATR}hv
            TxZb = 0x10,                // ZigBeeMv
            TxRat = 0x17,               // uATR}hv
            RxAt = ( 0x80 | TxAt ),     // ATR}h
            RxMst = 0x8a,               // fXe[^X
            RxZbSt = 0x8b,              // ZigBeeMXe[^X
            RxZb = ( 0x80 | TxZb ),     // ZigBeeMpPbg
            RxRat = ( 0x80 | TxRat ),   // uATR}h
            _top
        }

        // (remote) AT command response
        // command status.
        public const byte atstOk = 0;                   // OK
        public const byte atstError = 1;                // G[
        public const byte atstInvalidCommand = 2;       // ȃR}h
        public const byte atstInvCmnd = atstInvalidCommand;
        public const byte atstInvalidParameter = 3;     // ȃp[^
        public const byte atstInvPar = atstInvalidParameter;
                                                        // TxG[A܂́A
        public const byte atstTxFailure = 4;            // uR}hMs
        public enum atst : byte
        {
            Ok = 0,                     // OK
            Error = 1,                  // G[
            InvalidCommand = 2,         // ȃR}h
            InvCmnd = InvalidCommand,
            InvalidParameter = 3,       // ȃp[^
            InvPar = InvalidParameter,
                                        // TxG[A܂́A
            TxFailure = 4,              // uR}hMs
            _top
        }

        // ZigBee transmit status
        // delivery status.
        public const byte zbdvSuccess = 0x00;
        public const byte zbdvMacAckFailure = 0x01;
        public const byte zbdvCcaFailure = 0x02;
        public const byte zbdvInvalidDestinationEndpoint = 0x15;
        public const byte zbdvNetworkAckFailure = 0x21;
        // ...
        public enum zbdv : byte
        {
            Success = 0x00,
            MacAckFailure = 0x01,
            CcaFailure = 0x02,
            InvalidDestinationEndpoint = 0x15,
            NetworkAckFailure = 0x21,
            // ...
            _top
        }

        // ZigBee transmit status
        // discovery status.
        public const byte zbdcNoOverhead = 0x00;
        public const byte zbdcAddress = 0x01;
        public const byte zbdcRoute = 0x02;
        public const byte zbdcAddressAndRoute = zbdcAddress | zbdcRoute;
        public const byte zbdcExtTimeout = 0x40;
        public enum zbdc : byte
        {
            NoOverhead = 0x00,
            Address = 0x01,
            Route = 0x02,
            AddressAndRoute = Address | Route,
            ExtTimeout = 0x40,
            _top
        }
    }

    public class XBlib
    {
        public enum RC : int    // result code.
        {
            Success = 0,
            InvalidDelimiter = -1000,   // f~^sł
            Timeout,                    // ^CAEg܂
            Toolong,                    // w蒷obt@̃TCY𒴂܂
            Overflow,
            InvalidPort,                // w肳ꂽ|[gJĂ܂
            InvalidSum,                 // `FbNTG[
            InApifault,                 // API[hɓ܂
            InvalidResponce,            // sȃX|Xł
            InvalidAtStatus,            // ATXe[^XG[
            TxZbFailed,            // ZigBeeMXe[^XG[
            DiffApiId,                  // ҂API IDł͂Ȃ
            DiffAtCmd,                  // ҂ATR}hł͂Ȃ
            ModuleNotFnd,               // W[܂
            NoCoordinator,              // R[fBl[^[܂
            OutOfRange,                 // p[^l傫܂
            InvalidSendStatus,          // MXe[^XG[
            TableFull,                  // e[uɋ󂫂Ȃ
            ToManyNode,                 // m[h̎gp\𒴂܂
            NotSelected,                // m[hIĂ܂
            CantOpenPort                // VA|[gI[vł܂
        }

        public struct NE // node entry.
        {
            public uint sh;         // 64bitAhX32bit
            public uint sl;         // 64bitAhX32bit
            public byte nah;        // 16bitlbg[NAhX8bit
            public byte nal;        // 16bitlbg[NAhX8bit
            public byte sn0;        // 64bitAhX
            public byte sn1;        // 64bitAhX
            public byte sn2;        // 64bitAhX
            public byte sn3;        // 64bitAhX
            public byte sn4;        // 64bitAhX
            public byte sn5;        // 64bitAhX
            public byte sn6;        // 64bitAhX
            public byte sn7;        // 64bitAhX
            public string ni;       // m[hID
        }

        private const int BUFFSIZE1 = 0x40;
        private const int BUFFSIZE2 = 0x80;

        private SerialPort serialPort1;
        private static Object lockObj = new Object();
        private byte bFID = 0;      // ݂̃t[ID

        //-----------------------------------------------------------------------------------------------
        // 
        //-----------------------------------------------------------------------------------------------
        public XBlib(SerialPort sp)
        {
            serialPort1 = sp;
            serialPort1.ReadTimeout = 4000;
            bFID = 0;
        }

        //-----------------------------------------------------------------------------------------------
        // 
        //-----------------------------------------------------------------------------------------------
        public static string MsgStr(RC rc)
        {
            switch (rc)
            {
            case RC.ToManyNode:
                return "m[h̎gp\𒴂܂";
            case RC.CantOpenPort:
                return "VA|[gI[vł܂";
            case RC.InApifault:
                return "API[hɓ܂ł";
            case RC.InvalidDelimiter:
                return "sȃf~^ł";
            case RC.InvalidPort:
                return "w肳ꂽ|[gJĂ܂";
            case RC.InvalidSum:
                return "`FbNTG[";
            case RC.Timeout:
                return "^CAEg܂";
            case RC.Toolong:
                return "Mobt@̃TCY𒴂܂";
            case RC.InvalidResponce:
                return "sȉł";
            case RC.InvalidAtStatus:
                return "sATR}hł";
            case RC.TxZbFailed:
                return "ZigBeeMɎs܂";
            case RC.ModuleNotFnd:
                return "W[܂";
            case RC.NoCoordinator:
                return "R[fBl[^܂";
            case RC.TableFull:
                return "e[uɋ󂫂Ȃ܂";
            case RC.OutOfRange:
                return "p[^l͈͂𒴂Ă܂";
            case RC.InvalidSendStatus:
                return "MANmbWG[";
            case RC.NotSelected:
                return "ΏۂIĂ";
            default:
                break;
            }
            return "`̃G[ł";
        }

        //-----------------------------------------------------------------------------------------------
        // Mobt@̃f[^L`FbN
        // y߂lzMobt@̃f[^1ȏFTrueA0FFalse
        //-----------------------------------------------------------------------------------------------
        public bool chkRxBuffer()
        {
            if (serialPort1.BytesToRead > 0) return true;
            else return false;
        }

        //-----------------------------------------------------------------------------------------------
        // ݂̃t[ID擾
        // y߂lz݂̃t[ID
        //-----------------------------------------------------------------------------------------------
        public byte fidCur()
        {
            return bFID;
        }

        //-----------------------------------------------------------------------------------------------
        // t[IDCNg
        // y߂lz݂̃t[ID
        //-----------------------------------------------------------------------------------------------
        private byte fidInc()
        {
            bFID++;
            if (bFID < 1 || 0xfe < bFID)
            {
                bFID = 1;
            }
            return bFID;
        }

        //-----------------------------------------------------------------------------------------------
        // APIt[𑗐M
        // yz@data FMf[^iMCWP[^AMABCC܂܂ȂyC[ĥ݁j
        //           len  Ff[^
        // y߂lzUgXe[^XiRC.Success̏ꍇAȊȌꍇ̓G[R[hj
        //-----------------------------------------------------------------------------------------------
        public RC txApiFrame(byte[] data,int len)
        {
            byte[] buffer = new byte[BUFFSIZE2];
            byte d;
            int i, off = 0;
            RC rc = RC.Success;

            data[1] = fidInc();                 // t[IDݒ

            buffer[0] = 0x7e;                   // X^[gf~^

            // t[ʐݒ
            d = (byte)((len >> 8) & 0xff);
            if (d == 0x7e || d == 0x7d || d == 0x11 || d == 0x13)   // GXP[v
            {
                buffer[1] = 0x7d;
                off = 1;
                d = (byte)(d ^ 0x20);
            }
            buffer[1 + off] = d;

            // t[ʐݒ
            d = (byte)(len & 0xff);
            if (d == 0x7e || d == 0x7d || d == 0x11 || d == 0x13)   // GXP[v
            {
                buffer[2 + off] = 0x7d;
                off++;
                d = (byte)(d ^ 0x20);
            }
            buffer[2 + off] = d;
            off += 3;

            // ʂnꂽt[f[^[Jobt@ɓ]
            for (i = 0; i < len; i++)
            {
                d = data[i];
                if (d == 0x7e || d == 0x7d || d == 0x11 || d == 0x13)   // GXP[v
                {
                    buffer[i + off] = 0x7d;
                    off++;
                    d = (byte)(d ^ 0x20);
                }
                buffer[i + off] = d;
            }

            // `FbNToCg̐ݒ
            d = 0;
            for (i = 0; i < len; i++) d += data[i];                 // t[f[^̃Tl߂
            d = (byte)(0xff - d);                                   // `FbNTl
            if (d == 0x7e || d == 0x7d || d == 0x11 || d == 0x13)   // GXP[v
            {
                buffer[len + off] = 0x7d;
                off++;
                d = (byte)(d ^ 0x20);
            }
            buffer[len + off++] = d;

            lock (lockObj)
            {
                try
                {
                    serialPort1.Write(buffer, 0, len + off);
                }
                catch (TimeoutException)
                {
                    rc = RC.Timeout;
                }
                catch (ArgumentException)
                {
                    rc = RC.Toolong;
                }
            }
            return rc;
        }
        //-----------------------------------------------------------------------------------------------
        // APIt[Măt[f[^̂bufferɃZbg
        // yz@bufferFMf[^i[oCgziQƌ^j
        // y߂lzf[^@}CiXl̏ꍇ̓G[R[h
        //-----------------------------------------------------------------------------------------------
        public RC rxApiFrame(ref byte[] buffer)
        {
            int i, len, d, sum = 0;

            // X^[gf~^M
            if ((d = readByte()) < 0)  return (RC)d;
            if (d != 0x7E) return RC.InvalidDelimiter;

            // t[M
            if ((d = readByte()) < 0) return (RC)d;
            len = d << 8;
            if ((d = readByte()) < 0) return (RC)d;
            len += d;

            // t[f[^M
            try
            {
                for (i = 0; i < len; i++)
                {
                    if ((d = readByte()) < 0)
                        return (RC)d;
                    buffer[i] = (byte)d;
                    sum += d;
                }
            }
            catch (IndexOutOfRangeException)
            {
                return RC.Toolong;
            }

            // `FbNTmF
            if ((d = readByte()) < 0)
                return (RC)d;
            if ((byte)(sum + d) != 0xFF)
                return RC.InvalidSum;

            return RC.Success;
        }

        //-----------------------------------------------------------------------------------------------
        // GXP[v1oCgM
        // y߂lzMf[^@}CiX̏ꍇ̓G[R[h
        //-----------------------------------------------------------------------------------------------
        private int readByte()
        {
            int d;
            try
            {
                d = serialPort1.ReadByte();     // 1oCgM
            }
            catch (TimeoutException)
            {
                d = (int)RC.Timeout;
            }
            catch (InvalidOperationException)
            {
                d = (int)RC.InvalidPort;
            }
            if (d == 0x7d)                      // ESCȂΎ̃f[^M
            {
                try
                {
                    d = serialPort1.ReadByte();
                }
                catch (TimeoutException)
                {
                    d = (int)RC.Timeout;
                }
                d = d ^ 0x20;
            }

            return d;
        }

        //===============================================================================================
        // R[fBl[^mF
        // yz@s Fm[hʎqi[ϐiQƌ^j
        // y߂lzUgXe[^XiRC.Success̏ꍇAȊȌꍇ̓G[R[hj
        //===============================================================================================
        public RC atNI(ref string s)
        {
            RC rc = RC.Success;
            byte[] buffer = new byte[BUFFSIZE1];

            serialPort1.DiscardInBuffer();              // Mobt@̃f[^j
                                                        // GXP[vgp
            byte[] ATAP = { 0x08, 0x01, (byte)'A', (byte)'P', 0x02 };
                                                        // ATAPR}hoiESCgpj
            if ((rc = txApiFrame(ATAP, ATAP.Length)) < 0) return rc;
                                                        // ʂM
            if ((rc = rxApiFrame(ref buffer)) < 0) return rc;

            byte[] ATNI = { 0x08, 0x01, (byte)'N', (byte)'I' };
                                                        // ATNIR}ho 
            if ((rc = txApiFrame(ATNI, ATNI.Length)) < 0) return rc;
                                                        // ʂM
            if ((rc = rxApiFrame(ref buffer)) < 0) return rc;
            if (buffer[0] != 0x88) return RC.DiffApiId; // API ID`FbNi0x88 : ATR}hj
                                                        // ATR}h1ڃ`FbN
            if (buffer[2] != (byte)'N') return RC.DiffAtCmd;
                                                        // ATR}h2ڃ`FbN
            if (buffer[3] != (byte)'I') return RC.DiffAtCmd;
                                                        // ATR}hXe[^X`FbN
            if (buffer[4] != 0x00) return RC.InvalidAtStatus;
            int cnt;
            for (cnt = 0; cnt < 20; cnt++) if (buffer[cnt + 5] == 0) break;
                                                        // ATR}ĥݕɕϊ
            s = Encoding.ASCII.GetString(buffer, 5, cnt);
            return RC.Success;
        }

        //===============================================================================================
        // uATR}hsip[^pj
        // yӁzm[he[uo^̑Mɂ͎gpłȂ
        // yz@cmd  FATR}h
        //           node FMm[h̃m[he[ue
        // y߂lzG[R[h
        //===============================================================================================
        public RC sendRemoteAT(string cmd, NE node)
        {
            byte[] buff = new byte[32];
            buff[0] = 0x17;                             // uATR}h
            buff[1] = 0;                                // t[ID
            buff[2] = node.sn0;
            buff[3] = node.sn1;
            buff[4] = node.sn2;
            buff[5] = node.sn3;
            buff[6] = node.sn4;
            buff[7] = node.sn5;
            buff[8] = node.sn6;
            buff[9] = node.sn7;
            buff[10] = node.nah;
            buff[11] = node.nal;
            buff[12] = 0x02;
            buff[13] = (byte)cmd[0];
            buff[14] = (byte)cmd[1];
            return txApiFrame(buff, 15);
        }

        //===============================================================================================
        // uATR}hsiPoCgp[^pj
        // yӁzm[he[uo^̑Mɂ͎gpłȂ
        // yz@cmd  FATR}h
        //           node FMm[h̃m[he[ue
        //           prm  Fp[^
        // y߂lzG[R[h
        //===============================================================================================
        public RC sendRemoteAT(string cmd, NE node, byte prm)
        {
            byte[] buff = new byte[32];
            buff[0] = 0x17;                             // uATR}h
            buff[1] = 0;                                // t[ID
            buff[2] = node.sn0;
            buff[3] = node.sn1;
            buff[4] = node.sn2;
            buff[5] = node.sn3;
            buff[6] = node.sn4;
            buff[7] = node.sn5;
            buff[8] = node.sn6;
            buff[9] = node.sn7;
            buff[10] = node.nah;
            buff[11] = node.nal;
            buff[12] = 0x02;
            buff[13] = (byte)cmd[0];
            buff[14] = (byte)cmd[1];
            buff[15] = prm;
            return txApiFrame(buff, 16);
        }

        //===============================================================================================
        // uATR}hsi2oCgp[^pj
        // yӁzm[he[uo^̑Mɂ͎gpłȂ
        // yz@cmd  FATR}h
        //           node FMm[h̃m[he[ue
        //           prm  Fp[^
        // y߂lzG[R[h
        //===============================================================================================
        public RC sendRemoteAT(string cmd, NE node, ushort prm)
        {
            byte[] buff = new byte[32];
            buff[0] = 0x17;                             // uATR}h
            buff[1] = 0;                                // t[ID
            buff[2] = node.sn0;
            buff[3] = node.sn1;
            buff[4] = node.sn2;
            buff[5] = node.sn3;
            buff[6] = node.sn4;
            buff[7] = node.sn5;
            buff[8] = node.sn6;
            buff[9] = node.sn7;
            buff[10] = node.nah;
            buff[11] = node.nal;
            buff[12] = 0x02;
            buff[13] = (byte)cmd[0];
            buff[14] = (byte)cmd[1];
            buff[15] = (byte)(prm >> 8);
            buff[16] = (byte)(prm & 0xff);
            return txApiFrame(buff, 17);
        }

        //===============================================================================================
        // ZigBeeMv
        // yӁz@m[he[uo^̑Mɂ͎gpłȂ
        // yz@node FMm[h̃m[he[ue
        //         @data Ff[^
        //         @len@Ff[^
        // y߂lz<0 ̏ꍇʏG[R[hA >0̃t[ID
        //===============================================================================================
        public RC sendRequest(NE node, byte[] data, int len)
        {
            byte[] buff = new byte[32];
            buff[0] = 0x10;                             // ZigBeeMv
            buff[1] = 0;                                // t[ID
            buff[2] = node.sn0;
            buff[3] = node.sn1;
            buff[4] = node.sn2;
            buff[5] = node.sn3;
            buff[6] = node.sn4;
            buff[7] = node.sn5;
            buff[8] = node.sn6;
            buff[9] = node.sn7;
            buff[10] = node.nah;
            buff[11] = node.nal;
            buff[12] = 0;                               // u[hLXg͈
            buff[13] = 0;                               // IvV
            for (int i = 0; i < len; i++) buff[i + 14] = data[i];
            return txApiFrame(buff, 14 + len);
        }

        //===============================================================================================
        // ZigBeeMv
        // yӁz@m[he[uo^̑Mɂ͎gpłȂ
        // yz@node FMm[h̃m[he[ue
        //         @data Ff[^
        //         @len@Ff[^
        // y߂lz<0 ̏ꍇʏG[R[hA >0̃t[ID
        //===============================================================================================
        public RC sendRequest(NE node, string cmnd)
        {
            byte[] buff = new byte[32];
            buff[0] = 0x10;                             // ZigBeeMv
            buff[1] = 0;                                // t[ID
            buff[2] = node.sn0;
            buff[3] = node.sn1;
            buff[4] = node.sn2;
            buff[5] = node.sn3;
            buff[6] = node.sn4;
            buff[7] = node.sn5;
            buff[8] = node.sn6;
            buff[9] = node.sn7;
            buff[10] = node.nah;
            buff[11] = node.nal;
            buff[12] = 0;                               // u[hLXg͈
            buff[13] = 0;                               // IvV
            for (int i = 0; i < cmnd.Length; i++)
            {
                buff[i + 14] = (byte)cmnd[i];
            }
            return txApiFrame(buff, 14 + cmnd.Length);
        }
        //===============================================================================================
        // NDR}hNodeEntryɕϊ
        //===============================================================================================
        public bool neFromNd(ref NE ne, byte[] nd)
        {
            if (XBee.ftRxAt != nd[0]
///                    || fidCur() != nd[1]
                    || (byte)'N' != nd[2]
                    || (byte)'D' != nd[3]
                    || XBee.atstOk != nd[4]
                    )
            {
                return false;
            }
            ne.nah = nd[5];
            ne.nal = nd[6];
            ne.sh = ((uint)nd[7] << 24)
                    | ((uint)nd[8] << 16)
                    | ((uint)nd[9] << 8)
                    | (uint)nd[10]
                    ;
            ne.sl = ((uint)nd[11] << 24)
                    | ((uint)nd[12] << 16)
                    | ((uint)nd[13] << 8)
                    | (uint)nd[14]
                    ;
            ne.sn0 = nd[ 7];
            ne.sn1 = nd[ 8];
            ne.sn2 = nd[ 9];
            ne.sn3 = nd[10];
            ne.sn4 = nd[11];
            ne.sn5 = nd[12];
            ne.sn6 = nd[13];
            ne.sn7 = nd[14];
            ne.ni = "";
            for (int ii = 0; 0 != nd[15 + ii]; ii++)
            {
                ne.ni += (char)nd[15 + ii];
            }
            return true;
        }
    }
}
